=begin
#-----------------------------------------------------------------------------
Reverse_Back_Face_v1.9
(c) honoluludesktop 2010

Freely use, and abuse at your own risk. Fitness of use is 
not warranty implied.

The basic scanning procedure herein is by Chris Fuller, and 
"pick visable faces" is by Tomasz. 

10/15/2010 - v 1.0 Beta, first release.
10/30/2010 - v 1.2 Beta, edit component message.
10/31/2010 - v 1.3 Beta, user selected scanning increments.
11/22/2010 - v 1.7 Add faces from group, and components.
11/26/2010 - v 1.8 Retain last scanning rate.
12/19/2010 - v 1.9 Improve menu location
#-----------------------------------------------------------------------------
=end

def atelier_main_all_front_face_proc
	my_view = Sketchup.active_model.active_view
	Sketchup.active_model.selection.clear

	@definitions=Sketchup.active_model.definitions
	fov = Sketchup.active_model.active_view.field_of_view
	
	@rev_total=0				#total reversed face counter
	
	rev_default=[]				#initilize scanning rate, then retrieve from memory
	rev_default[0] = Sketchup.read_default "honoluludesktop","rev_default","Normal"
	
	#default=["Normal"]		#inputbox for scanning accuracy
	prompts=["Increments: "]
	list=["High|Normal|Medium"]
	result=UI.inputbox prompts, rev_default, list, "Scanning Accuracy"
	case result[0]
	when "High"
		ps=1						#every pixel
	when "Normal"
		ps=4						#every fourth pixel
	when "Medium"
		ps=12						#every twelve pixel
	end
										#save scanning rate in memory
	result = Sketchup.write_default "honoluludesktop","rev_default",result[0]
	
	if ps == 1						#run rate is for progress bar
		run_rate=20				#high
	elsif ps == 4
		run_rate=35				#normal
	else								#must be 12
		run_rate=60				#medium
	end
	
	Sketchup.set_status_text "Scaling Model:", SB_PROMPT
	
	w = my_view.vpwidth#SU window variables
	h = my_view.vpheight

#	f_temp = 120.0/fov		#f_temp is increment size =
	f_temp = 60.0/fov		#f_temp is increment size =
	i_temp = f_temp.to_i	#(120deg_max/fov)*2
	s_array = (i_temp.to_f/2).to_s.split(".")
	if s_array[1] == "0"  	#if no fraction must be even
		aa=i_temp+1			#aa is horiz increments
		bb=i_temp/2			#bb is starting point
	else								#must be odd
		aa=i_temp
		bb=(i_temp-1)/2
	end

	cw = w/aa					#horizontal pixels
	iw = cw*bb					#starting pixel
	ch = h/aa						#vertical pixels
	ih = ch*bb					#starting pixel

	my_view.field_of_view = 120.0	#max fov

	count_100 = 1				#initilize progress bar variables
	count_bar = ""
	@first_comp=true		#first time flag
	last_ent=""

	cw=cw/ps
	ch=ch/ps
	#procedure scanning pixels for entities
	hdt_scan_pixels(cw,iw,ch,ih,count_100,count_bar,last_ent,my_view,fov,ps,run_rate)
	
	Sketchup.set_status_text "Complete: Reverse [ #{@rev_total} ] Back Faces", SB_PROMPT	#completion status massage
	my_view.field_of_view = fov									#return original fov
	Sketchup.send_action "selectSelectionTool"				#set selection tool
end

def hdt_scan_pixels(cw,iw,ch,ih,count_100,count_bar,last_ent,my_view,fov,ps,run_rate)
cw.times do |x|				#begin scanning screen pixels, horizontal; tool by Chris Fuller
		ch.times do |y|		#vertical
			count_100 = count_100 +1			#progress bar at bottom of page
			if count_100 == run_rate
				count_bar = count_bar + "|"
				if count_bar.length == 100
					count_bar = ""
				end
				count_100 = 0
				Sketchup.set_status_text "Scanning for back faces: #{count_bar}", SB_PROMPT
			end
			xx=x*ps+iw	
			yy=y*ps+ih
			ph = my_view.pick_helper
			ph.do_pick(xx,yy)
			face=ph.picked_face
			hdt_process_ent(my_view,fov,xx,yy,face)
		end
	end
end

def hdt_process_ent(my_view,fov,xx,yy,face)
	my_view.field_of_view = fov	#put this here for aesthetic reasons
	if face!=nil 
		camera=my_view.camera			#proc variable by Tomasz
		ray=my_view.pickray xx,yy		#proc variable by Tomasz
		#proc to find back face to camera by Tomasz
		face, side=hdt_examineFace(face,camera,ray)#proc by Tomasz
		if side==false								#must be reversed face variable by Tomasz
			@rev_total=@rev_total+1
			face.reverse!
			my_view.invalidate
		end
	end
end

def hdt_examineFace(face, camera, ray)#proc by Tomasz
	model=Sketchup.active_model
	hit_point,family=model.raytest ray
	if hit_point != nil
			if family != nil
				if family.last.class==Sketchup::Face
					face=family.last
					family=hdt_findFaceBelow(hit_point, ray[1]) if (!face.visible? or !face.layer.visible?)
					if family != nil
						face=family.last
						trans=hdt_findGlobalTransformation(family)
						norm=face.normal
						norm.transform! trans
					end
				else
					norm=camera.direction.reverse
				end
			else
				norm=face.normal
			end
	end
	norm=camera.direction.reverse if norm==nil
	angle=camera.direction.angle_between norm
	return [face, angle>3.14159265/2.0]
end

	def hdt_findFaceBelow(hit_point, vector)#proc by Tomasz
		model=Sketchup.active_model
		new_origin, new_family=model.raytest [hit_point, vector]
		if new_origin != nil
				if new_family != nil
					if new_family.last.class==Sketchup::Face
						face=new_family.last
						if (!face.visible? or !face.layer.visible?)
							family=findFaceBelow(new_origin, vector)
						else
							family=new_family
						end
					end
				end
		end
		return family
	end

def hdt_findGlobalTransformation(parents)#proc by Tomasz
	trans=Geom::Transformation.new
	parents.each do |parent|
		trans=trans*parent.transformation if parent.class != Sketchup::Face
	end
	return trans
end

unless file_loaded? File::basename(__FILE__)
	ile_path_name = Sketchup.find_support_file("plugins")
	file_path_name=Sketchup.find_support_file "sketchup.exe", "" 
	file_path_name=File.dirname(file_path_name)+"/Plugins/hdt_folder/"

	select_tb = UI::Toolbar.new("Reverse Faces") 
	cmd4 = UI::Command.new("Reverse Faces") { atelier_main_all_front_face_proc } 
	cmd4.small_icon = file_path_name+"button_reverse_16.png" 
	cmd4.large_icon = file_path_name+"button_reverse_24.png" 
	cmd4.tooltip = "Reverse Faces" 
	select_tb.add_item(cmd4) 
	
	add_separator_to_menu("Tools")	
	UI.menu("Tools").add_item("Reverse back faces") { atelier_main_all_front_face_proc }
    file_loaded File::basename(__FILE__)
end